﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Logic
{
    public class CalculateFormula
    {
        public CalculateFormula(Dictionary<int , bool> AtomicDictionary , List<string> Formula)
        {
            this.Dictionary = AtomicDictionary;
            this.Counter = this.Dictionary.Where(a => a.Key >= 0).Select(a => a.Key).OrderBy(a => a).ToArray();
            this.Formula = Formula;
        }

        public Dictionary<int, bool> Dictionary { get; private set; }
        public List<string> Formula { get; private set; }
        public int[] Counter { get; private set; }

        private bool GetValue(string a)
        {
            if(a == "T")
                return true;
            else if(a == "B")
                return false;
            var t = int.Parse(a);            
            return Dictionary[t];
        }

        private bool IsAtomic(string a)
        {
            int c = -1;
            return int.TryParse(a, out c) || a=="B" || a=="T";            
        }

        public static Tuple<bool,string> IsTotology(string str)
        {
            str.Replace(" ", "");
            Tuple<bool,string> result = null;            
            if(str.Contains("="))
            {
                result = Detect(str.Replace('=', '>'));
                if(!result.Item1)
                    return result;
                var t = string.Join(">", str.Split('=').Reverse());
                result = Detect(t);                
            }
            else
            {
                result = Detect(str);
            }

            return result;
        }

        private static Tuple<bool,string> Detect(string str)
        {
            FormulaDetect fd = new FormulaDetect(str);
            CalculateFormula cf = new CalculateFormula(fd.AtomicDictionary, fd.FormulaStack);
            return new Tuple<bool, string>(cf.IsAlwaysTrue(),cf.CounterExpample());
        }

        private bool Calculate()
        {
            var clone = this.Formula.Select(a => a).ToList();
            for(int i = 1;i < clone.Count;i++)
            {
                if(!IsAtomic(clone[i]))
                {
                    if(clone[i]=="!")
                    {
                        clone[i] = Func(clone[i], clone[i - 1]).SS();
                        clone.RemoveAt(i - 1);
                    }
                    else
                    {
                        clone[i] = Func(clone[i], clone[i - 2], clone[i - 1]).SS();
                        clone.RemoveRange(i - 2, 2);
                    }
                    i = 0;
                }
            }

            if(clone.Count != 1)
                throw new Exception("یه چیز عجیب!");
            return GetValue(clone[0]);
        }

        
        private bool Func(string opt , string a , string b)
        {
            var aa = GetValue(a);
            var bb = GetValue(b);

            if(opt == ">")
                return !aa || bb;
            else if(opt == "|")
                return aa || bb;
            else if(opt == "&")
                return aa && bb;

            throw new NotSupportedException();
        }

        private bool Func(string opt , string a)
        {
            if(opt == "!")
                return !GetValue(a);
            throw new NotSupportedException();
        }

        public bool IsAlwaysTrue()
        {
            if(!Calculate())
                return false;
            while(!Count())
            {
                if(!Calculate())
                    return false;
            }

            return true;
        }

        public string CounterExpample()
        {
            return $"[{string.Join(",", Dictionary.Select(a => $"{a.Key}={a.Value}").ToArray())}]";
        }

        private bool Count()
        {
            if(Counter.Length == 0)
                return true;
            return Count(Counter.Length - 1);
        }

        private bool Count(int a)
        {
            if(a == 0)
            {
                if(Dictionary[Counter[a]])
                {
                    Dictionary[Counter[a]] = false;
                    return true;
                }
                else
                {
                    Dictionary[Counter[a]] = true;
                    return false;
                }
            }
            else
            {
                var b = Count(a - 1);
                if(b)
                {
                    if(Dictionary[Counter[a]])
                    {
                        Dictionary[Counter[a]] = false;
                        return true;
                    }
                    else
                    {
                        Dictionary[Counter[a]] = true;
                        return false;
                    }
                }
                else
                {
                    return b;
                }
            }            
        }



    }

    static class Extensions
    {
        public static string SS(this bool s)
        {
            if(s)
                return "T";
            else
                return "B";
        }
    }
}
