﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Logic
{
    public class FormulaDetect
    {
        public const string Then = ">";
        public const string Not = "!";
        public const string Or = "|";
        public const string And = "&";
        public const string But = "B";

        private static List<string> Adat = new List<string>
        {
            Then ,
            Not ,
            Or ,
            And
        };

        public Dictionary<int, bool> AtomicDictionary { get; private set; }
        public List<string> FormulaStack { get; private set; }

        public FormulaDetect(string Formula)
        {
            var result = ToPasvandiString(Formula);
            this.AtomicDictionary = result.Item2;
            this.FormulaStack = result.Item1;
        }

        static Tuple< List<string>,Dictionary<int, bool>> ToPasvandiString(string str)
        {
            StringTokenizer st = new StringTokenizer(str, ">!|&()", true);
            Stack<string> stack = new Stack<string>(st.Count);
            StringBuilder sb = new StringBuilder(str.Length);
            Dictionary<int, bool> dic = new Dictionary<int, bool>();
            int lastPer = 0;
            while(st.HasToken())
            {

                string s = st.NextToken();
                if(GetPeriorty(s) == -1) //not operator
                {
                    int a = -1;                    
                    if(int.TryParse(s, out a) &&!dic.ContainsKey(a))
                        dic.Add(a, false);
                    sb.Append(s + " ");
                }
                else if(s == "(")
                {
                    stack.Push(s);
                    lastPer = 0;
                }
                else if(s == ")")
                {
                    string ss = stack.Pop();
                    while(ss != "(")
                    {
                        sb.Append(ss + " ");
                        ss = stack.Pop();
                    }
                    if(stack.Count > 0)
                    {
                        s = stack.Pop();
                        lastPer = GetPeriorty(s);
                        stack.Push(s);
                    }
                }
                else if(GetPeriorty(s) >= lastPer)
                {
                    lastPer = GetPeriorty(s);
                    stack.Push(s);
                }
                else
                {
                    while(stack.Count > 0 && stack.Peek()!="(")
                    {
                        sb.Append(stack.Pop() + " ");
                    }
                    stack.Push(s);
                    lastPer = 0;
                }
            }

            while(stack.Count > 0)
            {
                sb.Append(stack.Pop() + " ");
            }


            return new Tuple<List<string>, Dictionary<int, bool>>( sb.ToString().Split(new char[] { ' ' },StringSplitOptions.RemoveEmptyEntries).ToList() , dic);

        }

        static int GetPeriorty(string c)
        {
            if(">".Contains(c.ToString()))
                return 1;
            else if("&|".Contains(c.ToString()))
                return 2;
            else if("!".Contains(c.ToString()))
                return 3;
            else if("()".Contains(c))
                return 0;

            else return -1;
        }

    }

    class StringTokenizer
    {
        string str;
        string sep;
        bool consep;
        Queue<string> q = new Queue<string>(100);
        public StringTokenizer(string Value, string Seperator, bool ContainSeperator = false)
        {
            str = Value;
            sep = Seperator;
            consep = ContainSeperator;
            Token();
        }

        private void Token()
        {
            StringBuilder sb = new StringBuilder(100);
            foreach(char c in str)
            {
                if(sep.Contains(c.ToString()))
                {
                    if(sb.ToString().Length > 0)
                        q.Enqueue(sb.ToString());
                    sb.Remove(0, sb.ToString().Length);
                    if(consep)
                        q.Enqueue(c.ToString());
                }
                else
                {
                    sb.Append(c);
                }
            }
            if(sb.Length > 0)
                q.Enqueue(sb.ToString());
        }

        public int Count { get { return q.Count; } }

        public bool HasToken()
        {
            return q.Count > 0;
        }

        public string NextToken()
        {
            return q.Dequeue();
        }

    }
}
